//     Author: Maurizio Loreti, aka MLO or (HAM) I3NOO
//     Work:   University of Padova - Department of Physics
//             Via F. Marzolo, 8 - 35131 PADOVA - Italy
//     Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102
//     EMail:  loreti@pd.infn.it
//     WWW:    http://www.pd.infn.it/~loreti/mlo.html
//
// -------------------------------------------------------------------
//
// Maze generation: main program
//
// -------------------------------------------------------------------
//
// $Id: main.cxx,v 1.1 2001/10/05 11:57:40 loreti Exp $

#include <iostream>
#include <stdexcept>
#include <string>
#include <sstream>
#include <cstdlib>
#include "maze.hh"

using std::cerr;

namespace {
  unsigned long readArg(
    const std::string in,
    unsigned long in_min,
    unsigned long in_max
  ) {
    unsigned long retVal;
    char *pc;

    retVal = strtoul(in.c_str(), &pc, 0);
    if (*pc != '\0') {
      throw std::runtime_error(in + ": invalid input");
    } else if (retVal < in_min   ||   retVal > in_max) {
      std::ostringstream s;
      s << in << ": value should be between " << in_min
        << " and " << in_max;
      throw std::range_error(s.str());
    } else {
      return retVal;
    }
  }
}

int main(
  int   argc,
  char *argv[]
) {
  unsigned long width, height, method;

  if (argc < 2   ||   argc > 4) {
    cerr << "Usage:\t" << argv[0] << " <width> [<height> [<method>]]\n";
    cerr << "\t<method> is 1 for DFS, 2 for Prim, 3 for Kruskal;\n";
    cerr << "\t<height> defaults to <width>, <method> to DFS.\n";
    return EXIT_FAILURE;
  }

  try {
    width  =                    readArg(argv[1], 3, 40);
    height = argc < 3 ? width : readArg(argv[2], 3, 40);
    method = argc < 4 ? 1     : readArg(argv[3], 1, 3);

    maze m(width, height);
    switch (method) {
      case 1:
        m.build(maze::DFS, maze::VERTICAL);
        break;
      case 2:
        m.build(maze::PRIM, maze::VERTICAL);
        break;
      case 3:
        m.build(maze::KRUSKAL, maze::VERTICAL);
        break;
    }
    m.print();
  }

  catch (std::exception & e) {
    cerr << "EXCEPTION - " << e.what() << std::endl;
    return EXIT_FAILURE;
  }
  catch (...) {
    cerr << "UNKNOWN EXCEPTION\n";
    return EXIT_FAILURE;
  }
  return EXIT_SUCCESS;
}
